import { Router } from "express";
import {
  ApplicationType,
  JobType,
  TypedRequest,
  USER_TYPE,
} from "../config/types";
import {
  authMiddleware,
  candidateMiddleware,
  organizationMiddleware,
} from "../middlewares/auth";
import Application from "../models/Application";
import Auth from "../models/Auth";
import Jobs from "../models/Job";
import ResumeAPI from "../utilities/apis/resume";

const router = Router();

router.get(
  "/",
  authMiddleware,
  async (req: TypedRequest<{ userId: string }, null>, res) => {
    try {
      const user = await Auth.findOne({ userId: req.query.userId });

      let jobs;

      if (user.userType === USER_TYPE.CANDIDATE) {
        jobs = await Jobs.find()
          .populate({
            path: "applications",
            select: ["candidate", "status"],
          })
          .populate({ path: "organization" });
      } else {
        jobs = await Jobs.find({ organization: req.query.userId }).populate({
          path: "applications",
          populate: {
            path: "candidate",
            select: [
              "name",
              "contacts",
              "dateOfBirth",
              "profilePicture",
              "resume",
              "resumeData",
            ],
          },
        });
      }

      return res.json({ jobs, success: true });
    } catch (error) {
      return res.json({ error, success: false });
    }
  }
);

router.post(
  "/",
  authMiddleware,
  organizationMiddleware,
  async (req: TypedRequest<{ userId: string }, JobType>, res) => {
    try {
      const newJob = new Jobs({ ...req.body, organization: req.query.userId });
      const job = await newJob.save();
      return res.json({ success: true, job });
    } catch (error) {
      return res.json({ success: false, error });
    }
  }
);

router.put(
  "/",
  authMiddleware,
  organizationMiddleware,
  async (req: TypedRequest<{ userId: string }, JobType>, res) => {
    try {
      const job = await Jobs.findByIdAndUpdate(req.body._id, {
        $set: req.body,
      });
      return res.json({ success: true, job });
    } catch (error) {
      return res.json({ success: false, error });
    }
  }
);

router.delete(
  "/",
  authMiddleware,
  organizationMiddleware,
  async (req: TypedRequest<{ userId: string }, { jobId: string }>, res) => {
    try {
      await Jobs.deleteOne({
        organization: req.query.userId,
        _id: req.body.jobId,
      });
      return res.json({ success: true });
    } catch (error) {
      return res.json({ success: false });
    }
  }
);

router.put(
  "/apply",
  authMiddleware,
  candidateMiddleware,
  async (
    req: TypedRequest<
      { userId: string },
      { application: ApplicationType; resumeUrl: string }
    >,
    res
  ) => {
    try {
      const { application, resumeUrl } = req.body;
      const job = await Jobs.findById(application.job);

      const data: any = await ResumeAPI.getResumeScores({
        user_id: req.query.userId,
        resume_url: resumeUrl,
        primary_skills: job.primarySkills,
        secondary_skills: job.secondarySkills,
        job_desc: job.description,
      });

      const score: ApplicationType["score"] = {
        primary: data.primary_score,
        primatyMatch: data.primary_match,
        secondary: data.secondary_score,
        secondaryMatch: data.secondary_match,
        similarity: data.similarity,
        total: data.primary_score + data.secondary_score + data.similarity,
      };

      const newApplication = new Application({ ...application, score });
      const _application = await newApplication.save();
      job.applications.push(_application.id);
      await job.save();
      return res.json({
        success: true,
        applicationId: _application.id,
      });
    } catch (error) {
      return res.json({ success: false, error });
    }
  }
);

module.exports = router;
